<?php

namespace CityBeach\CatchFeeder\Model;

use CityBeach\CatchFeeder\Api\ProductSummaryInterface;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Helper\Product;

class ProductSummary implements ProductSummaryInterface
{
    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    private $productRepository;

    /**
     * @var Magento\CatalogInventory\Api\StockRegistryInterface
     */
    private $stockRegistry;

    /**
     * @var \Magento\Catalog\Helper\Product
     */
    private $productHelper;

    /**
     * @var \Magento\Catalog\Model\Product\Gallery\ReadHandler
     */
    private $galleryReadHandler;

    /**
     * @param \Magento\Framework\ObjectManagerInterface $objectManager
     * @param \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry
     * @param \Magento\CatalogInventory\Model\Stock\StockItemRepository $stockItemRepository
     */
    public function __construct(
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Catalog\Helper\Product $productHelper,
        \Magento\Catalog\Model\Product\Gallery\ReadHandler $galleryReadHandler,
        \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry
    ) {
        $this->productRepository = $productRepository;
        $this->productHelper = $productHelper;
        $this->galleryReadHandler = $galleryReadHandler;
        $this->stockRegistry = $stockRegistry;
    }

    /**
     * Load Block data collection by given search criteria
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param \Magento\Framework\Api\SearchCriteriaInterface $criteria
     * @return array
     */
    public function getList(\Magento\Framework\Api\SearchCriteriaInterface $criteria)
    {
        $products = $this->productRepository->getList($criteria);
        $productSummaries = array();


        /* @var $product ProductInterface */
        foreach ($products->getItems() as $product) {
          // filling the product details
            $id = $product->getId();
            $typeInstance = $product->getTypeInstance();
            $productSummary = array(
                'id' => $id,
                'sku' => $product->getSku(),
                'name' => $product->getName(),
                'price' => $product->getPrice(),
                'weight' => $product->getWeight(),
                'status' => $product->getStatus(),
                'visibility' => $product->getVisibility(),
                'type' => $product->getTypeId(),
                'created' => $product->getCreatedAt(),
                'updated' => $product->getUpdatedAt(),
                'category_ids' => $product->getCategoryIds(),
            );
            $attributes = array();
            foreach ($product->getData() as $key => $value) {
                if (!empty($value) || strpos($key, '_') !== 0) {
                    switch ($key) {
                        case 'image':
                            $attributes[$key] = $this->productHelper->getImageUrl($product);
                            break;

                        case 'small_image':
                            $attributes[$key] = $this->productHelper->getSmallImageUrl($product);
                            break;

                        case 'thumbnail':
                            $attributes[$key] = $this->productHelper->getThumbnailUrl($product);
                            break;

                        default:
                            $attributes[$key] = $value;
                            break;
                    }
                }
            }

            // attaching the media gallery
            $this->galleryReadHandler->execute($product);
            // access the gallery
            $images = [];
            $gallery = $product->getMediaGalleryImages()->getItems();
            foreach ($gallery as $image) {
              $images[] = $image->getUrl();
            }
            $attributes['gallery'] = $images;

            $productSummary['attributes'] = $attributes;
            if ($product->getTypeId() === 'configurable') {
                $used = $typeInstance->getUsedProducts($product);
                $related = array();
                foreach ($used as $use) {
                    $related[] = $use->getId();
                }
                $productSummary['related'] = $related;
            }
            elseif ($product->getTypeId() === 'grouped') {
              $associatedIds = $typeInstance->getAssociatedProductIds($product);
              $related = array();
              foreach ($associatedIds as $associatedId) {
                $related[] = $associatedId;
              }
              $productSummary['related'] = $related;
            }
            elseif ($product->getTypeId() === 'bundle') {
                $childrenIds = $typeInstance->getChildrenIds($product->getId());
                $related = array();
                foreach ($childrenIds as $childId) {
                    $related[] = $childId;
                }
                $productSummary['related'] = $related;
            }

            $stock = $this->queryStock($product);
            $productSummary['stock'] = $stock;
            $productSummary['link'] = $this->productHelper->getProductUrl($product);

            $productSummaries[$id] = $productSummary;
        }

        return $productSummaries;
    }

    /**
     * Count product summary for products matching the specified criteria.
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param \Magento\Framework\Api\SearchCriteriaInterface $criteria
     * @return array
     */
    public function getCount(\Magento\Framework\Api\SearchCriteriaInterface $criteria)
    {
        $products = $this->productRepository->getList($criteria);
        return [[
          'product_summary_count' => $products->getTotalCount(),
        ]];
    }

    private function queryStock($product) {
      $productStock = $this->stockRegistry->getStockItem($product->getId());
      return $productStock->getQty();
    }

}
